/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //                                                 EMS.h
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  Copyright (c) 2025 Carlo Marcheggiani 

    EMS.h is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************************************************************/

// Defines for Engine Management System
#define UP        0    // for Flaps and Landing Gear postition
#define DOWN      1  // for Flaps and Landing Gear postition
#define TRUE      1  // general flag
#define FALSE     0 // general flag
#define GEARFLAG  0 // Gear flag bit on IO Module (IN1)
#define FLAPFLAG  1 // Flap flag bit on IO Module (IN2)

// Special colors
const uint16_t	RA8875_DARK_GREEN            = 0x0500;

// Aircraft parameters definition //
#define	VSF		63
#define	VS		76
#define	VF		113
#define VX		100
#define VY		110
#define	VM		172
#define	VNE		275
#define	VMAX	250
#define	FS		300
#define	EGTNOR		850.0
#define	EGTMAX		880.0
#define	EGTDELTA	50.0

// Engine parameters limimits
#define	CHTMIN		70.0
#define	CHTNOR		120.0
#define	CHTMAX		135.0
#define	CHTDELTA	10.0

#define OILPMIN		0.8
#define	OILPNOR		5.0
#define	OILPMAX		7.0
#define OILPFS		8.0

#define	OILTZER		30.0
#define OILTMIN		50.0
#define	OILTNOR		130.0
#define	OILTMAX		140.0
#define OILTFS		150.0

#define RPMZER		0
#define	RPMNOR		5500
#define	RPMMAX		5800
#define	RPMFS		  6000

#define	MAPZER		0
#define	MAPMAX		30.0
#define	MAPFS		  33.0

#define	VLTMIN		12.0  // Low limit with RPM=0
#define	VLTMIN1		13.6  // Low limit with RPM>0
#define	VLTCHG		14.0	// Vcharge batteria PbCa (Spark600)
#define	VLTMAX		15.0

#define	AMPMIN		-10.0
#define	AMPMAX		15.0

#define	OATMIN  	-20.0
#define	OATMAX  	40.0

#define	CATMIN  	5.0
#define	CATMAX  	45.0

#define FPZER   	0.0
#define FPMIN   	0.15
#define FPMAX   	0.5
#define FPFS		  0.7

#define	FFZER		  0.0
#define	FFMIN		  10.0
#define	FFMAX		  20.0
#define	FFFS		  30.0

#define	FUELZERLEVEL	0
#define	FUELMINLEVEL	7
#define	FUELNORLEVEL	9
#define	FUELFSLEVEL		27

#define	MINFUELQUANTITY	10
#define	MAXFUELQUANTITY	72
#define	MAXFUELDELTA	  6	  // Differenza massima tra carburante disponibile calcolato e carburante disponibile misurato
#define	MAXFUELDIFF		  10	// Differenza tra Left e Right per Tank Switch Advise
#define FUELMINBURNT    5   // Minimo consumo carburante

#define MAXPRESDELTA    5   // Max pressure difference between manifoldPressure e pressure

#define BALLMAX         5   // Massimo offset pallina

#ifndef _BV
  #define _BV(X)			(1<<X)
#endif

// Message class
//#define	NOP				0
//#define	ALARM			1
//#define	WARNING		2
//#define ADVISOR		3

// EFIS Alarm flags
#define ALM_NUMBER		22

#define ALM_LOASI		1UL
#define ALM_HIASI		2UL
#define ALM_OILP		4UL
#define ALM_OILT		8UL
#define ALM_FP			16UL
#define ALM_FF			32UL
#define ALM_CHT			64UL
#define ALM_RPM			128UL

#define ALM_EGT			256UL
#define ALM_VOLT		512UL
#define ALM_AMP			1024UL
#define ALM_CAT			2048UL
#define ALM_OAT			4096UL
#define ALM_FUEL		8192UL
#define ALM_MAP			16384UL
#define ALM_LFOOT		32768UL

#define ALM_RFOOT		65536UL
#define ALM_WARMUP		131072UL
#define ALM_SWTOLTANK	262144UL
#define ALM_SWTORTANK	524288UL
#define	ALM_GEAR		1048576UL
#define	ALM_FLAP		2097152UL

// MGL Avionics RDAC
#define RDAC_MSGS_TIMEOUT 200     // Number of loops without RDAC messages
#define RDAC_CAN_ID       32      // CAN ID of RDAC bit 4-11
#define VREF              5.023   // ADC Voltage Reference (extimated for better readings precision)
#define ADCMAX            4095    // Maximum ADC raw value
#define AMPOFFS1          2.5     // Current sensor voltage at zero ampere
#define AMPOFFS2          2.5     // Current sensor voltage at zero ampere
#define AMPVPA1           0.037   // Current sensor voltage per ampere
#define AMPVPA2           0.037   // Current sensor voltage per ampere
#define FPOFFSET          0.0 //0.0097  // Fuel pressure sensor offset
#define FPGAIN            1.0     // FUel pressure gain

// RELAY Board Input Unit
#define RELAY_MSGS_TIMEOUT 200     // Number of loops without RDAC messages

// VDO temperature sender coefficients (not used)
#define VDO150_A  3.193
#define VDO150_B	-61.72
#define VDO150_C	302.2
#define VDO120_A  -0.796	
#define VDO120_B	-20.39
#define VDO120_C	178.8

